<?php

namespace Drupal\snippet_manager\Plugin\SnippetVariable;

use Drupal\Core\Form\FormStateInterface;
use Drupal\snippet_manager\SnippetVariableBase;

/**
 * Provides layout region variable type.
 *
 * @SnippetVariable(
 *   id = "layout_region",
 *   title = @Translation("Layout region"),
 *   category = @Translation("Other"),
 * )
 */
class LayoutRegion extends SnippetVariableBase {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    $default_label = $this->configuration['label'] ?:
      ucfirst(trim(str_replace('_', ' ', $this->getName())));

    $form['label'] = [
      '#title' => $this->t('Label'),
      '#type' => 'textfield',
      '#default_value' => $default_label,
      '#required' => TRUE,
    ];

    $delta = max(abs($this->configuration['weight']), 50);
    $form['weight'] = [
      '#type' => 'number',
      '#min' => -$delta,
      '#max' => $delta,
      '#default_value' => $this->configuration['weight'],
      '#title' => $this->t('Weight'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'label' => '',
      'weight' => 0,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $cached_regions = &drupal_static('snippet_layout_regions');
    return render($cached_regions[$this->getSnippet()->id()][$this->getName()]);
  }

}
