<?php

namespace Drupal\snippet_manager;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Base class for snippet variables.
 */
abstract class SnippetVariableBase extends PluginBase implements SnippetVariableInterface {

  use StringTranslationTrait;

  /**
   * The snippet to which this variable is attached.
   *
   * @var \Drupal\snippet_manager\SnippetInterface
   */
  protected $snippet;

  /**
   * The name of the variable.
   *
   * @var string
   */
  protected $name;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->setConfiguration($configuration);
  }

  /**
   * {@inheritdoc}
   */
  public function getType() {
    return $this->t('String');
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['message'] = [
      '#markup' => $this->t('This plugin has no configurable options.'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->setConfiguration($form_state->getValues());
  }

  /**
   * {@inheritdoc}
   */
  public function getOperations() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = NestedArray::mergeDeep(
      $this->defaultConfiguration(),
      $configuration
    );
  }

  /**
   * {@inheritdoc}
   */
  public function setConfigurationValue($key, $value) {
    $this->configuration[$key] = $value;
  }

  /**
   * Sets snippet for the variable plugin.
   *
   * @param \Drupal\snippet_manager\SnippetInterface $snippet
   *   The snippet.
   */
  public function setSnippet(SnippetInterface $snippet) {
    $this->snippet = $snippet;
  }

  /**
   * Returns the snippet.
   *
   * @return \Drupal\snippet_manager\SnippetInterface
   *   The snippet.
   */
  public function getSnippet() {
    return $this->snippet;
  }

  /**
   * Sets variable name.
   *
   * @param string $name
   *   The name of the variable.
   */
  public function setName($name) {
    $this->name = $name;
  }

  /**
   * Returns variable name.
   *
   * @return string
   *   Variable name.
   */
  public function getName() {
    return $this->name;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    return [];
  }

}
